@@ -57,19 +57,19 @@ static GSList *device_list = NULL;
 static struct connman_device *find_device(const char *interface)
 {
 	GSList *list;
+	int index;
 
 	if (interface == NULL)
 		return NULL;
 
+	index = connman_inet_ifindex(interface);
+	if (index < 0)
+		return NULL;
+
 	for (list = device_list; list; list = list->next) {
 		struct connman_device *device = list->data;
-		const char *device_interface;
 
-		device_interface = connman_device_get_interface(device);
-		if (device_interface == NULL)
-			continue;
-
-		if (g_str_equal(device_interface, interface) == TRUE)
+		if (connman_device_get_index(device) == index)
 			return device;
 	}
 
@@ -286,11 +286,9 @@ char *__connman_udev_get_devtype(const char *ifname)
 {
 	struct udev_device *device;
 	const char *devtype;
-	char syspath[128];
 
-	snprintf(syspath, sizeof(syspath) - 1, "/sys/class/net/%s", ifname);
-
-	device = udev_device_new_from_syspath(udev_ctx, syspath);
+	device = udev_device_new_from_subsystem_sysname(udev_ctx,
+							"net", ifname);
 	if (device == NULL)
 		return NULL;
 
@@ -304,6 +302,62 @@ done:
 	return NULL;
 }
 
+char *__connman_udev_get_mbm_devnode(const char *ifname)
+{
+	struct udev_device *device, *parent, *control;
+	const char *driver, *devpath1, *devpath2;
+	char *devnode = NULL;
+	unsigned int i;
+
+	device = udev_device_new_from_subsystem_sysname(udev_ctx,
+							"net", ifname);
+	if (device == NULL)
+		return NULL;
+
+	parent = udev_device_get_parent(device);
+	if (parent == NULL)
+		goto done;
+
+	driver = udev_device_get_driver(parent);
+	if (g_strcmp0(driver, "cdc_ether") != 0)
+		goto done;
+
+	parent = udev_device_get_parent_with_subsystem_devtype(device,
+							"usb", "usb_device");
+	if (parent == NULL)
+		goto done;
+
+	devpath1 = udev_device_get_devpath(parent);
+
+	for (i = 0; i < 64; i++) {
+		char sysname[16];
+
+		snprintf(sysname, sizeof(sysname) - 1, "ttyACM%d", i);
+
+		control = udev_device_new_from_subsystem_sysname(udev_ctx,
+							"tty", sysname);
+		if (control == NULL)
+			continue;
+
+		parent = udev_device_get_parent_with_subsystem_devtype(control,
+							"usb", "usb_device");
+		if (parent == NULL)
+			continue;
+
+		devpath2 = udev_device_get_devpath(parent);
+
+		if (g_strcmp0(devpath1, devpath2) == 0) {
+			devnode = g_strdup(udev_device_get_devnode(control));
+			break;
+		}
+	}
+
+done:
+	udev_device_unref(device);
+
+	return devnode;
+}
+
 int __connman_udev_init(void)
 {
 	GIOChannel *channel;
